<?php

namespace Otys\OtysPlugin\Controllers\Admin;

use Otys\OtysPlugin\Includes\Core\Hooks;
use Otys\OtysPlugin\Includes\OtysApi\OtysApi;
use Otys\OtysPlugin\Includes\Core\Cache;
use Otys\OtysPlugin\Includes\Core\Webhooks;
use Otys\OtysPlugin\Models\Admin\AdminSettingsModel;

class AdminSettingsController extends AdminBaseController
{
    /**
     * The pagename
     * Pagename gets used in every setting registration
     */
    private const PAGE_NAME = 'otys_settings';

    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Run functions before class initialisation.
     * function gets called when registering controller at routes.
     *
     * @return void
     */
    public static function beforeInit(): void
    {
    
        $otysApi = new OtysApi;

        // $otysApi->documents->createDefaultDocuments();

        // Initialise the settings on page load
        Hooks::addAction('admin_init', __class__, 'initSettings', 10, 0);

        // Option validation on update
        Hooks::addAction('pre_update_option_otys_option_api_key', __class__, 'checkApiKEy', 10, 2);

        // Option validation on update
        Hooks::addAction('pre_update_option_otys_option_website', __class__, 'resetCache', 10, 2);
    }

    /**
     * Reset cache callback
     *
     * @return void
     */
    public static function resetCache($new, $old)
    {
        Cache::deleteAll();
        add_settings_error('otys_messages', 'otys_message', __('All plugin related cache has been removed', 'otys-jobs-apply'), 'updated');

        return $new;
    }

    /**
     * Callback when API KEY changes
     *
     * @param string $new
     * @param string $old
     * @return string
     */
    public static function checkApiKEy($new, $old)
    {
        if ($new !== $old) {
            $otysApi = new OtysApi($new);
            
            if (!$otysApi->session) {
                add_settings_error('otys_messages', 'otys_message', __('Invalid API key given', 'otys-jobs-apply'), 'error');
            } else {
                Webhooks::deleteWebhooks($otysApi->session);
                AdminSettingsModel::createDefaultDocuments($new);
                Webhooks::registerWebhooks($otysApi->session);
                Cache::deleteAll();
                add_settings_error('otys_messages', 'otys_message', __('All plugin related cache has been removed', 'otys-jobs-apply'), 'updated');
            }
        }

        return $new;
    }

    /**
     * Init settings for admin settings page
     *
     * @return void
     */
    public static function initSettings(): void
    {
        $otysApi = new OtysApi();

        $check = $otysApi->check();

        /*
            API Section
        */
        // Add API Section to this page
        add_settings_section(
            'otys_api_section',
            __('Api settings', 'otys-jobs-apply'),
            function () {
            },
            self::PAGE_NAME
        );

        // register a new field in the "otys_api_section" section, inside the "otys_settings" page
        register_setting(self::PAGE_NAME, 'otys_option_api_key');
        add_settings_field(
            'otys_option_api_key',
            __('API key', 'otys-jobs-apply'),
            [__class__, 'textField'],
            self::PAGE_NAME,
            'otys_api_section',
            [
                'option_name' => 'otys_option_api_key',
                'option_section' => 'otys_api_section',
                'description' => __('Enter the API key provided by OTYS.', 'otys-jobs-apply')
            ]
        );

        /**
         * If current SESSION ID is valid show the rest of the settings
         */
        if ($check) {
            // Register new setting for otys_settings page
            register_setting(self::PAGE_NAME, 'otys_option_website');
            add_settings_field(
                'otys_option_website',
                __('Website', 'otys-jobs-apply'),
                [__class__, 'websiteRadio'],
                self::PAGE_NAME,
                'otys_api_section',
                [
                    'option_name' => 'otys_option_website',
                    'option_section' => 'otys_api_section',
                    'description' => __('This decides which website environment will be used for the Wordpress plugin', 'otys-jobs-apply')
                ]
            );

            /*
                Recaptcha
            */
            // Add API Section to this page
            add_settings_section(
                'otys_option_recaptcha_section',
                __('Recaptcha settings', 'otys-jobs-apply'),
                function () {
                },
                self::PAGE_NAME
            );

            // register a new field in the "otys_option_recaptcha_section" section, inside the "otys_settings" page
            register_setting(self::PAGE_NAME, 'otys_option_recaptcha_site_key');
            add_settings_field(
                'otys_option_recaptcha_site_key',
                __('Site key', 'otys-jobs-apply'),
                [__class__, 'textField'],
                self::PAGE_NAME,
                'otys_option_recaptcha_section',
                [
                    'option_name' => 'otys_option_recaptcha_site_key',
                    'option_section' => 'otys_recaptcha_section',
                    'description' => __('This site key is used in the HTML code your site serves to users.', 'otys-jobs-apply')
                ]
            );

            // register a new field in the "otys_option_recaptcha_section" section, inside the "otys_settings" page
            register_setting(self::PAGE_NAME, 'otys_option_recaptcha_secret_key');
            add_settings_field(
                'otys_option_recaptcha_secret_key',
                __('Secret Key', 'otys-jobs-apply'),
                [__class__, 'textField'],
                self::PAGE_NAME,
                'otys_option_recaptcha_section',
                [
                    'option_name' => 'otys_option_recaptcha_secret_key',
                    'option_section' => 'otys_recaptcha_section',
                    'description' => __('This secret key is used for communication between your site and reCAPTCHA.', 'otys-jobs-apply')
                ]
            );

            // register a new field in the "otys_option_recaptcha_section" section, inside the "otys_settings" page
            register_setting(self::PAGE_NAME, 'otys_option_recaptcha_threshold');
            add_settings_field(
                'otys_option_recaptcha_threshold',
                __('Threshold', 'otys-jobs-apply'),
                [__class__, 'selectScoreRecaptcha'],
                self::PAGE_NAME,
                'otys_option_recaptcha_section',
                [
                    'option_name' => 'otys_option_recaptcha_threshold',
                    'option_section' => 'otys_recaptcha_section',
                    'description' => __('reCAPTCHA learns by seeing real traffic on your site. For this reason, scores in a staging environment or soon after implementing may differ from production. As reCAPTCHA v3 doesn\'t ever interrupt the user flow, you can first run reCAPTCHA without taking action and then decide on thresholds by looking at your traffic in the admin console. By default, you can use a threshold of 0.5.', 'otys-jobs-apply')
                ]
            );

            /*
                Vacancies section
            */
            add_settings_section(
                'otys_vacancies_section',
                __('Vacancy Overview', 'otys-jobs-apply'),
                function () {
                },
                self::PAGE_NAME
            );

            // Register new setting for otys_settings page
            register_setting(self::PAGE_NAME, 'otys_option_vacancies_per_page', 'object');

            // register a new field in the "otys_vacancies_section" section, inside the "otys_settings" page
            add_settings_field(
                'otys_option_vacancies_per_page',
                __('Vacancies per page', 'otys-jobs-apply'),
                [__class__, 'textField'],
                self::PAGE_NAME,
                'otys_vacancies_section',
                [
                    'option_name' => 'otys_option_vacancies_per_page',
                    'class' => 'small-text',
                    'option_section' => 'otys_vacancies_section',
                    'description' => __('The amount of vacancies displayed per page on the vacancy overview.', 'otys-jobs-apply')
                ]
            );

            // Register new setting for otys_settings page
            register_setting(self::PAGE_NAME, 'otys_option_vacancies_filters_match_criteria');

            // register a new field in the "otys_vacancies_section" section, inside the "otys_settings" page
            add_settings_field(
                'otys_option_vacancies_filters_match_criteria',
                __('Match criteria filters', 'otys-jobs-apply'),
                [__class__, 'matchCriteriaSelect'],
                self::PAGE_NAME,
                'otys_vacancies_section',
                [
                    'option_name' => 'otys_option_vacancies_filters_match_criteria',
                    'class' => 'small-text',
                    'option_section' => 'otys_vacancies_section',
                    'description' => __('The match criteria candidates are able to filter on at the vacancy overview.', 'otys-jobs-apply')
                ]
            );

            // Register new setting for otys_settings page
            register_setting(self::PAGE_NAME, 'otys_option_vacancies_list_match_criteria_labels');

            // register a new field in the "otys_vacancies_section" section, inside the "otys_settings" page
            add_settings_field(
                'otys_option_vacancies_list_match_criteria_labels',
                __('Match criteria labels', 'otys-jobs-apply'),
                [__class__, 'matchCriteriaSelect'],
                self::PAGE_NAME,
                'otys_vacancies_section',
                [
                    'option_name' => 'otys_option_vacancies_list_match_criteria_labels',
                    'class' => 'small-text',
                    'option_section' => 'otys_vacancies_section',
                    'description' => __('The match criteria labels shown on the vacancy overview.', 'otys-jobs-apply')
                ]
            );

            /*
                Vacancies section
            */
            add_settings_section(
                'otys_vacancy_detail_section',
                __('Vacancy Detail', 'otys-jobs-apply'),
                function () {
                },
                self::PAGE_NAME
            );

            // Register new setting for otys_settings page
            register_setting(self::PAGE_NAME, 'otys_option_vacancies_detail_match_criteria_labels');

            // register a new field in the "otys_vacancy_detail_section" section, inside the "otys_settings" page
            add_settings_field(
                'otys_option_vacancies_detail_match_criteria_labels',
                __('Match criteria labels', 'otys-jobs-apply'),
                [__class__, 'matchCriteriaSelect'],
                self::PAGE_NAME,
                'otys_vacancy_detail_section',
                [
                    'option_name' => 'otys_option_vacancies_detail_match_criteria_labels',
                    'class' => 'small-text',
                    'option_section' => 'otys_vacancy_detail_section',
                    'description' => __('The match criteria labels shown on the vacancy detail.', 'otys-jobs-apply')
                ]
            );

            /*
                Mails
            */
            add_settings_section(
                'otys_mail_section',
                __('Mails', 'otys-jobs-apply'),
                function () {
                    $templatePath = OTYS_PLUGIN_ADMIN_TEMPLATE_URL . '/admin-settings/admin-settings-mails.php';

                    load_template($templatePath, false);
                },
                self::PAGE_NAME
            );

            // Register new setting for otys_settings page
            register_setting(self::PAGE_NAME, 'otys_option_document_template_apply_notify_candidate');

            // register a new field in the "otys_vacancy_detail_section" section, inside the "otys_settings" page
            add_settings_field(
                'otys_option_document_template_apply_notify_candidate',
                __('Candidate applied (existing)', 'otys-jobs-apply'),
                [__class__, 'documentSelect'],
                self::PAGE_NAME,
                'otys_mail_section',
                [
                    'option_name' => 'otys_option_document_template_apply_notify_candidate',
                    'class' => 'small-text',
                    'option_section' => 'otys_mail_section',
                    'description' => __('Email when an existing candidate applied for a job', 'otys-jobs-apply')
                ]
            );

            // Register new setting for otys_settings page
            register_setting(self::PAGE_NAME, 'otys_option_document_template_apply_notify_new_candidate');

            // register a new field in the "otys_vacancy_detail_section" section, inside the "otys_settings" page
            add_settings_field(
                'otys_option_document_template_apply_notify_new_candidate',
                __('Candidate applied (new)', 'otys-jobs-apply'),
                [__class__, 'documentSelect'],
                self::PAGE_NAME,
                'otys_mail_section',
                [
                    'option_name' => 'otys_option_document_template_apply_notify_new_candidate',
                    'class' => 'small-text',
                    'option_section' => 'otys_mail_section',
                    'description' => __('Email when a new candidate applied for a job.', 'otys-jobs-apply')
                ]
            );

            // Register new setting for otys_settings page
            register_setting(self::PAGE_NAME, 'otys_option_document_template_open_apply_notify_candidate');

            // register a new field in the "otys_vacancy_detail_section" section, inside the "otys_settings" page
            add_settings_field(
                'otys_option_document_template_open_apply_notify_candidate',
                __('Candidate open application', 'otys-jobs-apply'),
                [__class__, 'documentSelect'],
                self::PAGE_NAME,
                'otys_mail_section',
                [
                    'option_name' => 'otys_option_document_template_open_apply_notify_candidate',
                    'class' => 'small-text',
                    'option_section' => 'otys_mail_section',
                    'description' => __('Email when candidate did an open application.', 'otys-jobs-apply')
                ]
            );

            // Register new setting for otys_settings page
            register_setting(self::PAGE_NAME, 'otys_option_document_template_apply_confirm_email');

            // register a new field in the "otys_vacancy_detail_section" section, inside the "otys_settings" page
            add_settings_field(
                'otys_option_document_template_apply_confirm_email',
                __('Candidate confirm email', 'otys-jobs-apply'),
                [__class__, 'documentSelect'],
                self::PAGE_NAME,
                'otys_mail_section',
                [
                    'option_name' => 'otys_option_document_template_apply_confirm_email',
                    'class' => 'small-text',
                    'option_section' => 'otys_mail_section',
                    'description' => __('When a candidate applies with a known email adres, the candidate will receive an email with a code, the code needs to be filled in at the last step of the application proces to continue the application.', 'otys-jobs-apply')
                ]
            );

            // Register new setting for otys_settings page
            register_setting(self::PAGE_NAME, 'otys_option_document_template_apply_notify_consultant');

            // register a new field in the "otys_vacancy_detail_section" section, inside the "otys_settings" page
            add_settings_field(
                'otys_option_document_template_apply_notify_consultant',
                __('Vacancy owner application notification', 'otys-jobs-apply'),
                [__class__, 'documentSelect'],
                self::PAGE_NAME,
                'otys_mail_section',
                [
                    'option_name' => 'otys_option_document_template_apply_notify_consultant',
                    'class' => 'small-text',
                    'option_section' => 'otys_mail_section',
                    'description' => __('Notification to vacancy owner when a candidate applied on a vacancy.', 'otys-jobs-apply')
                ]
            );
        }
    }

    /**
     * Document select
     *
     * @param mixed $args
     * @return void
     * @since 1.0.0
     */
    public static function documentSelect($args): void
    {
        $otysApi = new OtysApi();

        // Default values
        $args = wp_parse_args(
            $args,
            array(
                'class' => 'regular-text'
            )
        );

        $args['documents'] = $otysApi->documents->getList();

        $templatePath = OTYS_PLUGIN_ADMIN_TEMPLATE_URL . '/wp-settings-api/cb-select-document.php';

        load_template($templatePath, false, $args);
    }

    /**
     * Matchcriteria select callback
     */
    public static function matchCriteriaSelect($args)
    {
        $otysApi = new OtysApi();

        // Default values
        $args = wp_parse_args(
            $args,
            array(
                'class' => 'regular-text'
            )
        );

        $args['match_criteria'] = $otysApi->matchCriteria->getList();

        $templatePath = OTYS_PLUGIN_ADMIN_TEMPLATE_URL . '/wp-settings-api/cb-select-match-criteria.php';

        load_template($templatePath, false, $args);
    }

    /**
     * Matchcriteria select callback
     */
    public static function websiteRadio($args)
    {
        $otysApi = new OtysApi();

        // Default values
        $args = wp_parse_args(
            $args,
            array(
                'class' => 'regular-text'
            )
        );

        $args['options'] = $otysApi->getWebsites();
        $args['options'][0] = __('No multisite', 'otys-jobs-apply');
        ksort($args['options']);

        $templatePath = OTYS_PLUGIN_ADMIN_TEMPLATE_URL . '/wp-settings-api/cb-radio.php';

        load_template($templatePath, false, $args);
    }

    /**
     * Recaptcha callback
     */
    public static function selectScoreRecaptcha($args)
    {
        $option = get_option($args['option_name']);

        $args = wp_parse_args(
            $args,
            array(
                'class' => 'regular-text'
            )
        );

        echo '<select name="' . esc_attr($args['option_name']) . '">';
        for ($i = 0.1; $i < 1; $i = $i + 0.1) {
            echo ' <option value="' . esc_attr($i) . '" ' . (isset($option) && $option == $i ? esc_attr('selected') : '') . '>' . esc_html($i) . '</option>';
        }

        echo '</select>';

        if (isset($args['description'])) {
            echo '<p class="description">' . wp_kses_post($args['description']) . '</p>';
        }
    }
}
