<?php

namespace Otys\OtysPlugin\Controllers;

use Otys\OtysPlugin\Includes\OtysApi\OtysApi;
use Otys\OtysPlugin\Includes\Core\Webhooks;
use Otys\OtysPlugin\Includes\Core\Cache;
use Otys\OtysPlugin\Includes\Core\Routes;

final class WebhooksController extends BaseController
{
    public function __construct()
    {
        parent::__construct();
        $this->model->modulename = 'webhooks';
    }

    /**
     * Entry point for webhook callbacks
     *
     * @return void
     * @since 1.0.0
     */
    public function index()
    {
        // Retrieve sent data
        $data = file_get_contents('php://input');
        $request = json_decode($data, true);

        // Get action slug
        $method = isset($request['request']) ?  str_replace('Otys.Services.', '', $request['request']['method']) : 'none';
        $slug = sanitize_title($method);

        $webhook = Webhooks::get($slug);

        // Check if the webhook exists & is callable if so run the callable function
        if ($webhook !== false && is_callable($webhook['callback'])) {
            call_user_func(
                [$webhook["callback"][0], $webhook['callback'][1]],
                $request["request"]
            );

            $this->jsonResponse([
                'type' => 'success',
                'code' => 'webhook_no_response',
                'message' => 'Action was called but no response was given'
            ]);
        }

        // Webhook was not found
        $this->jsonResponse([
            'type' => 'error',
            'code' => 'webhook_action_not_found',
            'message' => 'The given action ' . $slug . ' does not exists.'
        ]);

        return;
    }

    /**
     * Filter transients by keyword
     *
     * @param array $transients
     * @param array $keywords
     * @return array
     */
    private function filter(array $transients, array $keywords): array
    {

        foreach ($transients as $key => $transient) {
            $match = false;

            foreach ($keywords as $keyword) {
                // Check if the transient contains the vacancy uid
                if (strpos($transient['value'], $keyword)) {
                    $match = true;
                }
            }
            
            if ($match === false) {
                unset($transients[$key]);
            }
        }

        return $transients;
    }

    
    /**
     * Webhook when vacancy is updated
     *
     * @param array $request
     * @return void
     * @since 1.0.0
     */
    public static function webhookVacancyUpdate(array $request)
    {
        $otysApi = new OtysApi();

        // Init vars
        $args = $request['args'];
        $identifier = $args[1];
        $data = $args[2];
        $keywords = [];
        $transients = [];
        
        // Check what keywords to search for when doing a update
        $keywords[] = $identifier;


        $keywordTransiensts = Cache::getList([
            'keywords' => $keywords,
            'includeExpired' => false
        ]);

        
        if (!empty($keywordTransiensts)) {
            foreach ($keywordTransiensts as $transient) {
                $transients[] = $transient;
            }
        }

        /**
         * Check if vacancy has a external application set if not
         * we'll set the external application url for the vacancy
         */
        // TODO Need to finish this after there are new fields in OTYS for this. See support ticket 287372
        // $vacancy = $otysApi->vacancies->get($identifier, [
        //     'uid' => 1,
        //     'title' => 1,
        //     'customApplyUrl' => 1,
        //     'customDetailUrl' => 1
        // ], false);

        // if (!empty($vacancy) && array_key_exists('customApplyUrl', $vacancy) && array_key_exists('customDetailUrl', $vacancy)) {
        //     if (empty($vacancy['customApplyUrl'])) {
        //         $vacancyDetailUrl = get_site_url() . Routes::get('vacancy-detail', [
        //             'name' => sanitize_title($vacancy['title']) . '-' . $vacancy['uid']
        //         ]);
        //     }
            
        //     if ($vacancy['customDetailUrl'] === NULL) {
        //         $vacancyDetailUrl = get_site_url() . Routes::get('vacancy-detail', [
        //             'name' => sanitize_title($vacancy['title']) . '-' . $vacancy['uid']
        //         ]);

        //         var_dump($vacancyDetailUrl);
        //     }
        // }

        /**
         * Check if current request contains a key published
         * If it does we assume the request was done from the
         * publication widget and the request could be changing
         * something about the publication. Therefore we'll refresh
         * all vacancies list requests
         */
        $publishCacheFound = false;
        foreach ($data as $key => $property) {
            if (strpos($key,'publish') !== false) {
                $publishCacheFound = true;
                break;
            }
        }

        if ($publishCacheFound) {
            $publishTransients = Cache::search('vacancyservice_getlist');
            
            if (!empty($publishTransients)) {
                foreach ($publishTransients as $transient) {
                    $transients[] = $transient;
                }
            }
        }

        $responses = Cache::refresh($transients);

        /**
         * Field names that trigger the vacancy overview cache refresh
         */
        static::jsonResponse([
            'type' => 'success',
            'code' => 'webhook_vacancy_update',
            'message' => 'Deleted cache',
            'data' => [
                $responses
            ]
        ]);
    }

    /**
     * Webhook when match criteria is updated
     *
     * @param array $request
     * @return void
     */
    public static function webhookMatchCriteriaUpdate(array $request): void
    {
        // Init vars
        $args = $request['args'];
        $keywords = [];

        if (isset($args[1])) {
            $identifier = $args[1];

            // Check what keywords to search for when doing a update
            if ($request['method'] === 'Otys.Services.MatchCriteriaService.update') {
                $ids = explode ('_', $identifier);
                $keywords[] = $ids[1]; // Use the option search
            } elseif ($request['method'] === 'Otys.Services.MatchCriteriaSettingsService.update') {
                $keywords[] = "C_{$identifier}";
                $keywords[] = "matchCriteria_{$identifier}";
            }
        }

        $transients = Cache::getList([
            'keywords' => $keywords,
            'includeExpired' => false
        ]);

        $transients = static::filter($transients, $keywords);
        $responses = Cache::refresh($transients);

        /**
         * Field names that trigger the vacancy overview cache refresh
         */
        static::jsonResponse([
            'type' => 'success',
            'code' => 'webhook_match_criteria_update',
            'message' => 'Deleted cache',
            'data' => [
                $responses
            ]
        ]);
    }

    /**
     * Webook when questionset is updated
     *
     * @param array $request
     * @return void
     */
    public function webhookQuestionsetUpdate(array $request): void
    {
        // Init vars
        $args = $request['args'];

        $keywords = [];

        if(isset($args[1]) && $questionsetData = $args[1]) {
            if (is_int($questionsetData)) {
                $keywords[] = filter_var($questionsetData, FILTER_CALLBACK, array('options' => [$this, 'sanitizeOUID']));
            } else {
                if (isset($questionsetData['questionId'])) {
                    $keywords[] = filter_var($questionsetData['questionId'], FILTER_CALLBACK, array('options' => [$this, 'sanitizeOUID']));
                }
                
                if (isset($questionsetData['setId'])) {
                    $keywords[] = filter_var($questionsetData['setId'], FILTER_CALLBACK, array('options' => [$this, 'sanitizeOUID']));
                }
            }
        }

        $transients = Cache::getList([
            'keywords' => $keywords,
            'includeExpired' => false
        ]);

        $transients = static::filter($transients, $keywords);

        $responses = Cache::refresh($transients);

        /**
         * Field names that trigger the vacancy overview cache refresh
        */
        static::jsonResponse([
            'type' => 'success',
            'code' => 'webhook_questionset_update',
            'message' => 'Deleted cache',
            'data' => [
                $responses
            ]
        ]);
    }

    /**
     * Webhook when CSM is updated
     *
     * @param array $request
     * @return void
     */
    public function webhookCsmUpdate(array $request)
    {
        // Init vars
        $args = $request['args'];
        $identifier = $args[1];
        $data = $args[2];
        $keywords = [];
        
        // Check what keywords to search for when doing a update
        $keywords[] = $identifier;

        $transients = Cache::getList([
            'keywords' => $keywords,
            'includeExpired' => false
        ]);

        $transients = static::filter($transients, $keywords);
        $responses = Cache::refresh($transients);

        /**
         * Field names that trigger the vacancy overview cache refresh
        */
        static::jsonResponse([
            'type' => 'success',
            'code' => 'webhook_csm_update',
            'message' => 'Deleted cache',
            'data' => [
                $responses
            ]
        ]);
    }
}
