<?php

/**
 * Base contains logic shared between all classes
 */

namespace Otys\OtysPlugin\Includes\Core;

abstract class Base
{
    /**
     * Store all intances, used to keep track of all instances created
     *
     * @since 1.0.0
     */
    private static $instances = array();

    /**
     * Provides access to a Single instance of a module using the singleton pattern
     *
     * @return self
     * @since 1.0.0
     */
    public static function getInstance(): self
    {
        $className = get_called_class();

        if (!isset(self::$instances[ $className ])) {
            self::$instances[ $className ] = new $className();
        }

        return self::$instances [ $className ];
    }

    /**
     * Get language based on current locale. Returns supported
     * language within OTYS
     *
     * @return string
     * @since 1.0.0
     */
    public static function getLanguage(string $locale = ''): string
    {
        if ($locale === "") {
            return "en";
        }

        $splitLocale = explode('_', $locale);
        $language = $splitLocale[0];

        $supportedLanguages = [
            'nl',
            'en',
            'de',
            'cs',
            'es',
            'fr'
        ];

        if (array_search($language, $supportedLanguages) !== false) {
            return $language;
        }

        return 'en';
    }

    /**
     * Sanitize UTM tags
     *
     * @param mixed $utmValue
     * @return mixed
     */
    public static function sanitizeUTM($utmValue)
    {
        if (is_array($utmValue)) {
            foreach ($utmValue as $utm => $value) {
                if (is_string($value)) {
                    $utmValue[$utm] = (string) preg_replace('/[^a-zA-Z0-9]/', '', $value);
                } else {
                    $utmValue[$utm] = '';
                }
            }

            return (array) $utmValue;
        } else {
            if (!is_string($utmValue)) {
                return '';
            }

            return (string) preg_replace('/[^a-zA-Z0-9]/', '', $utmValue);
        }
    }

    /**
     * Sanatize OUID
     *
     * @param mixed $uid
     * @return string
     */
    public static function sanitizeOUID($uid): string
    {
        if (is_string($uid)) {
            return preg_replace('/[^a-zA-Z0-9-_]/', '', $uid);
        }

        return '';
    }
}
