<?php

namespace Otys\OtysPlugin\Includes\Core;

use Otys\OtysPlugin\Includes\OtysApi\OtysApi;

class Webhooks extends Base
{
    /**
     * Stores list of all webhooks to be registered at OTYS
     */
    private static $webhooks = array();

    public function __construct()
    {
        Webhooks::add('VacancyService.update', ['\Otys\OtysPlugin\Controllers\WebhooksController', 'webhookVacancyUpdate']);
        Webhooks::add('VacancyService.updateEx', ['\Otys\OtysPlugin\Controllers\WebhooksController', 'webhookVacancyUpdate']);
        Webhooks::add('MatchCriteriaService.update', ['\Otys\OtysPlugin\Controllers\WebhooksController', 'webhookMatchCriteriaUpdate']);
        Webhooks::add('MatchCriteriaSettingsService.update', ['\Otys\OtysPlugin\Controllers\WebhooksController', 'webhookMatchCriteriaUpdate']);
        Webhooks::add('CandidateQuestionSetService.update', ['\Otys\OtysPlugin\Controllers\WebhooksController', 'webhookQuestionsetUpdate']);
        Webhooks::add('CsmService.setValue', ['\Otys\OtysPlugin\Controllers\WebhooksController', 'webhookCsmUpdate']);
    }

    /**
     * Add webhook to the list of to be registered webhooks
     *
     * @param  string $serviceCall  The OWS service call that should trigger the webhook
     * @param  string $webHookUrl The webhook URL that is being called if the OWS service call is executed
     * @param  bool $global Wheter the webhook gets trigged by all clients or only the webhook client where
     * true triggers the webhook for all users and false only triggers for the webhook user
     * @return void
     */
    public static function add(string $serviceCall, $callback, bool $klantGlobal = true): void
    {
        $slug = sanitize_title($serviceCall);
        $webhookCustomUrl = get_option('otys_option_webhook_url');

        if ($webhookCustomUrl !== false && WP_DEBUG) {
            self::$webhooks[$slug] = [
                'serviceCall' => $serviceCall,
                'webhookUrl' => $webhookCustomUrl,
                'callback' => $callback,
                'klantGlobal' => $klantGlobal
            ];
        } else {
            self::$webhooks[$slug] = [
                'serviceCall' => $serviceCall,
                'webhookUrl' => OTYS_PLUGIN_WEBHOOK_URL,
                'callback' => $callback,
                'klantGlobal' => $klantGlobal
            ];
        }
    }

    /**
     * Get list of webhooks added
     *
     * @return array
     * @since 1.0.0
     */
    public static function getList(): array
    {
        return self::$webhooks;
    }

    /**
     * Retrieve webhook from the list of webhooks
     *
     * @param  string $serviceCall  The OWS service call that should trigger the webhook
     * @param  string $webHookUrl The webhook URL that is being called if the OWS service call is executed
     * @param  bool $global Wheter the webhook gets trigged by all clients or only the webhook client where
     * true triggers the webhook for all users and false only triggers for the webhook user
     * @return void
     */
    public static function get(string $serviceCall)
    {
        $slug = sanitize_title($serviceCall);
        if (array_key_exists($slug, self::$webhooks)) {
            return self::$webhooks[$slug];
        }

        return false;
    }

    /**
     * Registers all webhooks with OTYS
     *
     * @return void
     */
    public static function registerWebhooks($session = false)
    {
        $otysApi = new OtysApi();

        $responses = [];

        $session = $session ? $session : $otysApi->getSession();

        foreach (self::$webhooks as $slug => $webhook) {
            $args = json_encode($webhook);

            // Check if the webhookUrl not is localhost since we don't want to register webhooks which are local
            $json = [
                "jsonrpc" => "2.0",
                "method" => "Otys.Services.WebhooksService.add",
                "params" => [
                    $session,
                    [
                        "serviceCall" => $webhook['serviceCall'],
                        "webhookUrl" => $webhook['webhookUrl'],
                        "klantGlobal" => $webhook['klantGlobal']
                    ]
                ],
                "id" => 1
            ];

            $responses[$slug] = $otysApi->post($json, false, false);
        }

        return $responses;
    }

    /**
     * Deletes all webhooks registered at OTYS
     *
     * @return void
     */
    public static function deleteWebhooks($session = false): void
    {
        $otysApi = new OtysApi();

        $session = $session ? $session : $otysApi->getSession();

        // First get a list of all registered webhooks
        $webhooks = $otysApi->post([
            "jsonrpc" => "2.0",
            "method" => "Otys.Services.WebhooksService.getList",
            "params" => [
                $otysApi->getSession()
            ],
            "id" => 1
        ], false, false);

        if ($webhooks) {
            $webhookCustomUrl = get_option('otys_option_webhook_url', OTYS_PLUGIN_WEBHOOK_URL);

            // Remove all webhooks individually
            foreach ($webhooks as $webhook) {
                // Check if the webhook contains the plugin's webhook url if so we will delete it
                if (
                    strpos($webhook['webhookUrl'], OTYS_PLUGIN_WEBHOOK_URL) !== false ||
                    strpos($webhook['webhookUrl'], $webhookCustomUrl) !== false
                ) {

                    $request = [
                        "jsonrpc" => "2.0",
                        "method" => "Otys.Services.WebhooksService.delete",
                        "params" => [
                            $otysApi->getSession(),
                            $webhook['uid']
                        ],
                        "id" => 1
                    ];

                    $otysApi->post($request, false, false);
                }
            }
        }
    }
}
