<?php

namespace Otys\OtysPlugin\Includes\OtysApi;

use Otys\OtysPlugin\Includes\Core\Routes;
use wpdb;

class CandidateQuestionSets extends OtysApiBase
{
    /**
     * Get list of candidate questionsets
     *
     * @return array
     */
    public function getList(): array
    {
        $owsRequestBody = [
            "jsonrpc" => "2.0",
            "method" => "Otys.Services.CandidateQuestionSetService.getListEx",
            "params" => [
                $this->otysApi->session,
                [
                    "what" => null,
                    "limit" => 25,
                    "offset" => 0,
                    "getTotalCount" => 1,
                    "sort" => [
                        "uid" => "DESC"
                    ],
                    "excludeLimitCheck" => true
                ]
            ],
            "id" => 29
        ];

        $candidateQuestions = $this->post($owsRequestBody, true);


        if ($candidateQuestions) {
            return $candidateQuestions;
        }

        return [];
    }

    /**
     * Get the candidate basic quesionset
     *
     * @return mixed
     */
    public function getVacancyQuestionsetId(string $vacancyUid)
    {
        $request = [
            "jsonrpc" => "2.0",
            "method" => "Otys.Services.QuestionSetService.getVacancyQsIdByVacancyOuid",
            "params" => [
                $this->otysApi->session,
                $vacancyUid
            ],
            "id" => 1
        ];

        return $this->post($request, true);
    }

    /**
     * Get specific candidate questionset
     *
     * @param  string $uid
     * @return mixed false if no question set is found else OWS object
     * @since  1.0.0
     */
    public function getCandidateQuestionset(string $uid = null)
    {
        if ($uid === null) {
            return [];
        }

        $request = [
            "jsonrpc" => "2.0",
            "method" => "Otys.Services.QuestionSetService.getDetail",
            "params" => [
                $this->otysApi->session,
                $uid
            ],
            "id" => 1
        ];

        $owsResult = $this->post($request, true);

        if ($owsResult) {
            return $owsResult;
        }

        return [];
    }

    /**
     * Get Open questionset
     *
     * @return void
     */
    public function getOpenQuestionset()
    {
        $request = [
            "jsonrpc" => "2.0",
            "method" => "Otys.Services.CandidateQuestionSetService.getListEx",
            "params" => [
                $this->otysApi->session,
                [
                    "what" => null,
                    "limit" => 1,
                    "offset" => 0,
                    "getTotalCount" => 1,
                    "sort" => [
                        "uid" => "DESC"
                    ],
                    "excludeLimitCheck" => true,
                    "condition" => [
                        "type" => "AND",
                        "items" => [
                            [
                                "type" => "OR",
                                "items" => [
                                    [
                                        "type" => "COND",
                                        "field" => "openEntrySet",
                                        "op" => "EQ",
                                        "param" => true
                                    ]
                                ]
                            ]
                        ]
                    ]
                ]
            ],
            "id" => 1
        ];

        $owsResult = $this->post($request);

        if (!empty($owsResult) && isset($owsResult['listOutput']) && isset($owsResult['listOutput'][0])) {
            return $owsResult['listOutput'][0];
        }

        return [];
    }

    /**
     * Get Questionset data based on vacancy id
     *
     * @param  int      $uid
     * @return array    Returns questionset as array
     * @since           1.0.0
     */
    public function getQuestionSet(int $questionsetId = 0): array
    {
        // Get questionset id based on vacancy id
        if ($questionsetId === 0) {
            return [];
        }

        // Get candidate qusetionset based on qusetionset id
        $candidateQuestionset = $this->getCandidateQuestionset($questionsetId);

        if (empty($candidateQuestionset)) {
            return [];
        }

        $currentPageNumber = get_query_var('page-number', 1);
        $currentPageNumber = $currentPageNumber === 0 ? 1 : $currentPageNumber;

        /*
             Create array of questionset data for the current page
        */
        $questionResult = array();

        foreach ($candidateQuestionset['Pages'] as $questionsetPageNumber => $questionsetPage) {
            // Setup page base information
            $questionResult[$questionsetPageNumber] = [
                'name' => isset($questionsetPage['name']) ? $questionsetPage['name'] : '',
                'pageTitle' => isset($questionsetPage['pageTitle']) ? $questionsetPage['pageTitle'] : '',
                'pageIntroText' => array_key_exists('pageIntroText', $questionsetPage) ? $questionsetPage['pageIntroText'] : '',
                'number' => $questionsetPageNumber
            ];

            $hasEmailField = false;

            // If there are no questions for this page add empty array
            if (empty($questionsetPage['Questions'])) {
                $questionResult[$questionsetPageNumber]['questions'] = [];
            }

            foreach ($questionsetPage['Questions'] as $questionid => $question) {
                // Skip when there's no type set
                if (!is_array($question) || !array_key_exists('type', $question)) {
                    continue;
                }

                // Template directory for forms used for field front end template

                
                // Define type since the type can be either determined using the actual type or the subtype, when a subtype is defined we use a subtype else just the type
                $type = strtolower(array_key_exists('subtype', $question) && !empty($question['subtype']) ? $question['subtype'] : $question['type']);
                
                // Define the field name attribute, this is used for the form post
                $fieldName = (array_key_exists('fieldName', $question) ? $question['fieldName'] : false);
                
                // If fieldname is email set has email true so we know the questionset includes a email adres
                if ($fieldName === 'Person.emailPrimary') {
                    $hasEmailField = true;
                }
                
                $htmlType = 'text';
                $templateDir = '/include-parts/forms/';

                switch ($type) {
                    case 'textfield':
                        $template = Routes::locateTemplate($templateDir . 'field-text');

                        if ($question['Validation']['phone'] === true || $question['Validation']['internationalPhone'] === true) {
                            $htmlType = 'tel';
                        } elseif ($question['Validation']['email'] === true) {
                            $htmlType = 'email';
                        }
                        break;

                    case 'textarea':
                        $template = Routes::locateTemplate($templateDir . 'field-textarea');
                        $htmlType = 'textarea';
                        break;

                    case 'motivation':
                        $template =  Routes::locateTemplate($templateDir . 'field-textarea');
                        $htmlType = 'textarea';
                        break;

                    case 'dateselect':
                        $template =  Routes::locateTemplate($templateDir . 'field-date');
                        $htmlType = 'date';
                        break;

                    case 'file':
                        $template =  Routes::locateTemplate($templateDir . 'field-file');
                        $htmlType = 'file';
                        break;

                    case 'multifile':
                        $template =  Routes::locateTemplate($templateDir . 'field-multifile');
                        $htmlType = 'multifile';
                        break;

                    case 'multiselect':
                        $template =  Routes::locateTemplate($templateDir . 'field-select');
                        $htmlType = 'multiselect';
                        break;

                    case 'select':
                        $template =  Routes::locateTemplate($templateDir . 'field-select');
                        $htmlType = 'select';
                        break;

                    case 'other':
                        $template =  Routes::locateTemplate($templateDir . 'field-checkbox');
                        $htmlType = 'checkbox';
                        break;

                    case 'checkbox':
                        $template =  Routes::locateTemplate($templateDir . 'field-checkbox');
                        $htmlType = 'checkbox';
                        break;

                    case 'radio':
                        $template =  Routes::locateTemplate($templateDir . 'field-radio');
                        $htmlType = 'radio';
                        break;


                    default:
                        $template =  Routes::locateTemplate($templateDir . 'field-text');
                        $htmlType = 'text';
                        break;
                }

                $questionResult[$questionsetPageNumber]['questions'][$questionid] = [
                    'id' => $questionid,
                    'name' => $questionid,
                    'fieldName' => $fieldName,
                    'template' => $template,
                    'type' => $htmlType,
                    'validation' => $question['Validation'],
                    'data' => $question
                ];

                if (array_key_exists('PossibleAnswersExt', $question)) {
                    $questionResult[$questionsetPageNumber]['questions'][$questionid]['answers'] = $question['PossibleAnswersExt'];
                }
            }

            // Add fake email field if the email does not exist in the questionset
            if ($hasEmailField === false && $questionsetPageNumber == 1) {
                $fakeQuestion = ['fake-email' => [
                    'id' => 'fake-email',
                    'name' => 'fake-email',
                    'fieldName' => 'Person.emailPrimary',
                    'template' => Routes::locateTemplate($templateDir . 'field-text'),
                    'type' => 'email',
                    'data' => [
                        'name' => __('E-mail', 'otys-jobs-apply'),
                        'page' => 1,
                        'orientation' => 'H',
                        'question' => __('E-mail', 'otys-jobs-apply'),
                        'type' => 'Personalia',
                        'fieldName' => 'email',
                        'subtype' => 'textField',
                        'documentTypeId' => 0,
                        'isKillerQuestion' => false,
                        'killApplication' => false,
                        'questionsListOrder' => 2,
                    ],
                    'validation' => [
                        'mandatory' => true,
                        'email' => true
                    ]
                ]];
                $questionResult[1]['questions'] = $fakeQuestion + $questionResult[1]['questions'];
            }
        }

        $returnData = [
            'uid' => $candidateQuestionset['uid'],
            'currentPage' => (int) $currentPageNumber,
            'name' => $candidateQuestionset['name'],
            'shortDescription' => $candidateQuestionset['shortDescription'],
            'pageTitle' => $candidateQuestionset['pageTitle'],
            'pageIntroText' => $candidateQuestionset['pageIntroText'],
            'pages' => $questionResult,
            'hasEmailField' => $hasEmailField,
            'errorTemplate' => Routes::locateTemplate($templateDir . 'field-error')
        ];

        return $returnData;
    }

    /**
     * Get known candidate questionset: Get's the known candidate questionset UID set in OTYS Go
     *
     * @return mixed
     */
    public function getKnownCandidateQuestionsetUid()
    {
        $owsCall = [
            "jsonrpc" => "2.0",
            "method" => "Otys.Services.CandidateQuestionSetService.getListEx",
            "params" => [
                $this->otysApi->session,
                [
                    "what" => null,
                    "limit" => 1,
                    "offset" => 0,
                    "getTotalCount" => 1,
                    "sort" => [
                        "uid" => "DESC"
                    ],
                    "excludeLimitCheck" => true,
                    "condition" => [
                        "type" => "AND",
                        "items" => [
                            [
                                "type" => "OR",
                                "items" => [
                                    [
                                        "type" => "COND",
                                        "field" => "knownCandidateSet",
                                        "op" => "EQ",
                                        "param" => true
                                    ]
                                ]
                            ]
                        ]
                    ]
                ]
            ],
            "id" => 1
        ];

        $owsJson = json_encode($owsCall);

        $questionResult = $this->post($owsJson, true);

        if ($questionResult && ($owsQuestionResult = $questionResult)) {
            return $owsQuestionResult['listOutput'][0]['uid'];
        }

        return false;
    }

    /**
     * Get list of Candidate questionsets
     *
     * @return mixed false if no questionsets are found else OWS object
     * @since  1.0.0
     */
    public function getCandidateQuestionList()
    {
        $request = [
            "jsonrpc" => "2.0",
            "method" => "Otys.Services.MultiService.execute",
            "params" => [
                [
                    [
                        "method" => "Otys.Services.CandidateQuestionSetService.getListEx",
                        "args" => [
                            $this->otysApi->session,
                            [
                                "what" => null,
                                "limit" => 25,
                                "offset" => 0,
                                "getTotalCount" => 1,
                                "sort" => [
                                    "uid" => "DESC"
                                ],
                                "excludeLimitCheck" => true
                            ]
                        ]
                    ]
                ]
            ],
            "id" => 1
        ];

        $candidate_questions = $this->post($request, true);

        if ($candidate_questions && ($ows_cadndidate_result = $candidate_questions)) {
            return $ows_cadndidate_result;
        }

        return false;
    }

    /**
     * Get post data from session with keys based on database fieldnames
     *
     * @param string Questionset ID
     * @return array
     */
    public function getFieldValues(string $identifier, string $questionsetId): array
    {
        // Lets use the postdata variable to store all post data from all different pages
        $postData = [];

        $pagesData = $this->getData($identifier);

        // If there is no page data return an empty array
        if (empty($pagesData) || !isset($pagesData['answers'])) {
            return [];
        }

        $questionset = $this->getQuestionSet($questionsetId);

        // Combine post data from all pages
        foreach ($pagesData['answers'] as $pageId => $pagesData) {
            // Loop through page answers and validate them
            foreach ($pagesData as $questionId => $questionValue) {
                // Check if there's a question value set and if the postdata exists in the questionset
                if ($questionValue !== '' && isset($questionset['pages'][$pageId]['questions'][$questionId])) {
                    $question = $questionset['pages'][$pageId]['questions'][$questionId];

                    if ($fieldName = $question['fieldName']) {
                        $postData[$fieldName] = $this->sanitizeInput($questionValue, $question['type']);
                    }
                }
            }
        }

        return $postData;
    }

    /**
     * Validate question answers
     *
     * @param string    $questions, array of questions including validators
     * @param mixed     $postData, post data in array name => value
     * @param boolean   $sanitize, sanitize output or not
     * @param array     $args, other arguments which can apply.
     *                  bool: uniqueEmail | Wheter to check if the email adres is unique
     * @return array    returns empty array when validation if successfull else returns array of errors
     */
    public function validate(array $questions, array $postData, bool $sanitize = true, array $args = []): array
    {
        // Return empty array when there are no errors
        $errors = [];
        $data = $postData;

        $args = array_replace_recursive([
            'uniqueEmail' => false
        ], $args);

        foreach ($questions as $question) {
            $value = (isset($postData[$question['id']])) ? $postData[$question['id']] : '';

            // Get validator options from the current question
            $validators = $question['validation'];

            // Skip current validation if the answer is empty and not required
            if ($value === '' && $validators['mandatory'] === false) {
                continue;
            }

            // Loop through all validators and return error array if validation failed
            foreach ($validators as $validator => $validation) {
                // Check if we should validate using the current validator
                if ($validation === false || $validation === null || $validation === 0) {
                    continue;
                }

                if ($validator === 'mandatory') {
                    if ($value == '' && $value == false) {
                        $errors[$question['id']][$validator] = __('This field is required', 'otys-jobs-apply');
                    }

                    if (isset($value['documents']) && empty($value['documents'])) {
                        $errors[$question['id']][$validator] = __('File upload is required', 'otys-jobs-apply');
                    }
                }

                if ($validator === 'firstLetterCapital') {
                    if (!preg_match('~^\p{Lu}~u', $value)) {
                        $errors[$question['id']][$validator] = __('First letter should be a capital letter', 'otys-jobs-apply');
                    }
                }

                if ($validator === 'allWordsCapital') {
                    if (!ctype_upper($value)) {
                        $errors[$question['id']][$validator] = __('All words should in capital', 'otys-jobs-apply');
                    }
                }

                if ($validator === 'allLettersCapital') {
                    if (!ctype_upper($value)) {
                        $errors[$question['id']][$validator] = __('All letters should in capital', 'otys-jobs-apply');
                    }
                }

                if ($validator === 'date') {
                    if (\DateTime::createFromFormat('Y-m-d H:i:s', $value) === false) {
                        $errors[$question['id']][$validator] = __('Value should be a valid date', 'otys-jobs-apply');
                    }
                }

                if ($validator === 'postCode') {
                    if ($value === '') {
                        $errors[$question['id']][$validator] = __('Postal code is invalid', 'otys-jobs-apply');
                    }
                }

                if ($validator === 'digitsMaxLen') {
                    if (preg_match_all("/[0-9]/", $value) > $validation) {
                        $errors[$question['id']][$validator] = __('To many digits', 'otys-jobs-apply');
                    }
                }

                if ($validator === 'email') {
                    if (!filter_var($value, FILTER_VALIDATE_EMAIL)) {
                        $errors[$question['id']][$validator] = __('Invalid email adres', 'otys-jobs-apply');
                    } else if ($args['uniqueEmail'] === true && !empty($this->otysApi->candidates->getCandidate($value))) {
                        $errors[$question['id']][$validator] = __('E-mail already exists', 'otys-jobs-apply');
                    }
                }

                if ($validator === 'validCharacters') {
                    if (!preg_match('/^[a-z0-9 .\-]+$/i', $value)) {
                        $errors[$question['id']][$validator] = __('Only numbers & letters are allowed', 'otys-jobs-apply');
                    }
                }

                if ($validator === 'phone') {
                    if (!is_numeric($value) || count_chars($value) < 8) {
                        $errors[$question['id']][$validator] = __('Invalid phonenumber', 'otys-jobs-apply');
                    }
                }

                if ($validator === 'internationalPhone') {
                    if (!is_numeric($value) || strpos($value, '+') === false || count_chars($value) < 8) {
                        $errors[$question['id']][$validator] = __('Please enter a international phonenumber', 'otys-jobs-apply');
                    }
                }

                if ($validator === 'maxDigits') {
                    if (preg_match_all("/[0-9]/", $value) > $validation) {
                        $errors[$question['id']][$validator] = __('To many digits', 'otys-jobs-apply');
                    }
                }

                if ($validator === 'validBirth') {
                    if (\DateTime::createFromFormat('Y-m-d H:i:s', $value) === false || strtotime($value) > strtotime('now')) {
                        $errors[$question['id']][$validator] = __('Value should be a valid date of birth', 'otys-jobs-apply');
                    }
                }

                if ($validator === 'documentType') {
                    //$errors[$question['id']][$validator] = 'documemntye';
                    // TODO: Currently not used

                    // var_dump($question);
                }
            }

            if ($sanitize === true && isset($data[$question['id']])) {
                $data[$question['id']] = $this->sanitizeInput($data[$question['id']], $question['type']);
            }
        }

        return [
            'errors' => $errors,
            'data' => $data
        ];
    }

    /**
     * Undocumented function
     *
     * @param mixed $value :    Value to sanitize
     * @param string $type :    Type of the value, Available types are:
     *                          text, email, tel
     * @return mixed
     */
    public function sanitizeInput($value, string $type = 'text')
    {
        switch ($type) {
            case 'text':
                return wp_filter_post_kses($value);
                break;

            case 'email':
                return sanitize_email($value);
                break;

            case 'tel':
                return preg_replace('/\D+/', '', $value);
                break;
        }

        return $value;
    }

    /**
     * Save post data in session 
     *
     * @param string $uid | Vacancy UID
     * @param int $questionsetId
     * @param array $postData | $_POST
     * @param int pageNumber
     * @return void
     */
    public function savePost(string $vacancyUid, int $questionsetId, array $postData = [], int $pageNumber = 1): void
    {
        $questionset = $this->getQuestionSet($questionsetId);

        // Loop through questions of the page
        foreach ($questionset['pages'][$pageNumber]['questions'] as $questionId => $question) {
            // check if the current question is filled in the postdata
            if (array_key_exists($questionId, $postData) && $postData[$questionId] != '') {
                // Save the current question value in the session

                if (is_array($postData[$questionId]) && array_key_exists('PossibleAnswersExt', $question['data'])) {
                    $_SESSION['questionset'][$vacancyUid]['answers'][$pageNumber][$questionId] = $postData[$questionId];
                } else if (!is_array($postData[$questionId])) {
                    $_SESSION['questionset'][$vacancyUid]['answers'][$pageNumber][$questionId] = htmlspecialchars($postData[$questionId], ENT_QUOTES);
                }
            } elseif ($question['type'] !== 'file' && $question['type'] !== 'multifile') {
                // If the current question is not set and type is not a file remove the value from the session
                unset($_SESSION['questionset'][$vacancyUid]['answers'][$pageNumber][$questionId]);
            }
        }
    }

    /**
     * Set data in session. Set data is used to store questionset information in the user $_SESSION
     * This is used to remember data between different pages and makes it possible to switch pages at any point
     *
     * @param string    $uid : Questionset UID
     * @param array     $value : Array of values to store
     * @return void
     */
    public function setData(string $uid = null, $value = []): void
    {
        if ($uid !== null) {
            $_SESSION['questionset'][$uid] = array_replace_recursive($_SESSION['questionset'][$uid], $value);
        }
    }

    public function addData(string $uid, int $page = 1, array $value): void
    {
        $currentValue = isset($_SESSION['questionset'][$uid]['answers'][$page]) ? $_SESSION['questionset'][$uid]['answers'][$page] : [];
        $_SESSION['questionset'][$uid]['answers'][$page] = array_replace_recursive($currentValue, $value);
    }

    /**
     * Get form data from SESSION based on uid
     *
     * @param string  $uid : Unique identifier
     * @return array
     */
    public function getData(string $uid): array
    {
        return (isset($_SESSION['questionset'][$uid])) ? $_SESSION['questionset'][$uid] : [];
    }

    /**
     * Check if in current session an application was done for a specific vacancy
     * @param string 
     */
    public function applicationCompleted(string $vacancyUid): bool
    {
        $data = $this->getData($vacancyUid);

        if (!empty($data) && isset($data['completed']) && $data['completed'] === true) {
            return true;
        }

        return false;
    }

    /**
     * Reset errors from session
     *
     * @param string $uid
     * @return void
     */
    public function resetErrors(string $uid)
    {
        $questionData = $this->getData($uid);

        if (!empty($questionData)) {
            unset($_SESSION['questionset'][$uid]['errors']);
        }
    }

    /**
     * Remove data from SESSION
     *
     * @param string $uid
     * @param string $identifier
     * @param boolean $innerKey
     * @param string $type
     * @return boolean
     */
    public function removeData(string $uid, string $identifier = '', $innerKey = false, $type = '')
    {
        $questionData = $this->getData($uid);

        if (isset($questionData['answers'])) {
            // If no specific identifier is given remove all data
            if ($identifier === '') {
                unset($_SESSION['questionset'][$uid]);
                return true;
            }

            // If identifier exists in the first level remove data with key equal to the identifier
            if (array_key_exists($identifier, $questionData)) {
                unset($_SESSION['questionset'][$uid][$identifier]);
                return true;
            }

            // If none of the above do an extensive search if questiondata is available
            foreach ($questionData['answers'] as $pageId => $pageAnswers) {
                if (array_key_exists($identifier, $pageAnswers)) {
                    if ($innerKey === false) {
                        // Handle data removal for when no keys are specified, which means all data should be removed from the question
                        $return = $_SESSION['questionset'][$uid];

                        unset($_SESSION['questionset'][$uid]['answers'][$pageId][$identifier]);
                    } else {
                        // Handle data removal for when there is a key specified
                        if ($type === 'multifile' || $type === 'file') {
                            unset($_SESSION['questionset'][$uid]['answers'][$pageId][$identifier]['documents'][$innerKey]);
                            // TODO Remove files from the OTYS server with OWS
                        } else {
                            unset($_SESSION['questionset'][$uid]['answers'][$pageId][$identifier][$innerKey]);
                        }
                    }
                }
            }

            return true;
        }

        return false;
    }

    /**
     * Get form data from SESSION based on uid
     *
     * @param string    $uid : Questionset UID
     * @return array
     */
    public function getPageData(string $uid, int $pageId): array
    {
        $data = $this->getData($uid);
        return isset($data['answers'][$pageId]) ? $data['answers'][$pageId] : [];
    }
}
