<?php

namespace Otys\OtysPlugin\Includes\OtysApi;

use Otys\OtysPlugin\Includes\OtysApi\OtysApiBase;

class Candidates extends OtysApiBase
{
    /**
     * Stores list of names of allowed user input. List is based on
     * https://ows.otys.nl/info/detail.php?service=Otys.Services.CandidateService
     *
     * @since 1.0.0
     */
    public $allowedUserInput = [];

    /**
     * Constructor
     *
     * @param class $otysApi
     *
     * @since 1.0.0
     */
    public function __construct($otysApi)
    {
        parent::__construct($otysApi);

        $this->allowedUserInput = [
            'Person.title',
            'Person.firstName',
            'Person.lastName',
            'Person.infix',
            'Person.initials',
            'Person.suffix',
            'Person.emailPrimary',
            'Person.birthdate',
            'Person.age',
            'Person.gender',
            'Person.nationalityCode',
            'Person.nationalityCodes',
            'Person.nationality',
            'Person.languageCode',
            'Person.skype',
            'Person.phoneMobile',
            'Person.AddressPrimary',
            'Person.AddressPrimary.address',
            'Person.AddressPrimary.city',
            'Person.AddressPrimary.postcode',
            'Person.AddressPrimary.stateCode',
            'Person.AddressPrimary.countryCode',
            'Person.AddressPrimary.country',
            'Person.PhoneNumbers.2',
            'Person.married',
            'Person.passPortNumber',
            'Person.passPortExpirationDate',
            'Person.bankAccountNumber',
            'Person.religion',
            'placeOfBirth',
            'phoneMobile',
            'availableAt',
            'Nationality',
            'promoText',
            'IdentityCardNumber',
            'SocialSecurityTaxNumber',
            'main_bank_iban_number',
            'IdentityCardExpirationDate',
            'website',
            'personalNr',
            'vatNumber',
            'greenCardExpirationDate',
            'greenCardNumber',
            'militaryHistory',
            'ownsCar',
            'workPermitExpirationDate',
            'workPermitNumber',
            'graduationDate',
            'age',
            'acceptTerms',
            'companyAddressCity',
            'companyWebsite',
            'imId',
            'handicapped',
            'willRelocate',
            'specificWorkplaceRequirement',
            'Linkedin',
            'hasActiveCustomerInLastWorkExperience',
            'ExtraPhoneNumbers',
            'EducationHistory',
            'EmploymentHistory',
            'SoftSkills',
            'ComputerSkills',
            'LanguageSkills',
            'ConfigurableOrganisationUnits',
            'Hobbies',
            'HrmItems',
            'References',
            'UtmTags',
            'BankAccounts',
            'Addresses',
            'noticeTerm',
            'driversLicence',
            'driversLicenceCopy',
            'driversLicenceNumber',
            'driversLicenceExpirationDate',
            'vatNr',
            'varType',
            'varCopy',
            'varExpirationDate',
            'vcaCopy',
            'vcaExpirationDate',
            'vcaNumber',
            'workContractCopy',
            'workContractNumber',
            'workContractEndDate',
            'workBaseLocationCountryCode',
            'workBaseLocation',
            'workBaseType',
            'numberOfKids',
            'socialSecurityTaxNumber',
            'matchCriteria_1',
            'matchCriteria_2',
            'matchCriteria_3',
            'matchCriteria_4',
            'matchCriteria_5',
            'matchCriteria_6',
            'matchCriteria_7',
            'matchCriteria_8',
            'matchCriteria_9',
            'matchCriteria_10',
            'matchCriteria_11',
            'matchCriteria_12',
            'matchCriteria_13',
            'matchCriteria_14',
            'matchCriteria_15',
            'matchCriteria_16',
            'matchCriteria_17',
            'matchCriteria_18',
        ];
    }

    /**
     * createCandidate
     *
     * @param  mixed $candidateData
     * @return mixed
     *
     * @since 1.0.0
     */
    public function createCandidate(array $candidateData)
    {
        if (empty($candidateData)) {
            return [];
        }

        // Build OWS based candidate data
        $candidateData = $this->buildCandidateData($candidateData);

        // Get information regarding this current api instance
        $apiInfo = $this->otysApi->getInfo();
        
        // Build default OWS data for creating the user
        $owsData = [
            "pointOfEntry" => "web",
            "varCopy" => null,
            "vcaCopy" => null,
            "currencyId" => 1,
            "statusId" => -1,
            "originSiteId" => $apiInfo['websiteId'],
            "source" => "website"
        ];

        // Add some extra data if terms has been accepted
        if (isset($candidateData) && isset($candidateData['acceptTerms']) && $candidateData['acceptTerms'] == "true") {
            $owsData['acceptTermsTs'] = date('Y/m/d h:i:s', time());
            $owsData['acceptTermsIp'] = is_string($_SERVER['REMOTE_ADDR']) ? preg_replace("/[^.:()a-zA-Z0-9\/]/", "", $_SERVER['REMOTE_ADDR']) : '';
        }

        // Add user input data to ows data
        $owsData = array_merge($owsData, $candidateData);

        // Build the actual ows call
        $owsCall = [
            "jsonrpc" => "2.0",
            "method" => "Otys.Services.CandidateService.add",
            "params" => [
                $this->otysApi->session,
                $owsData
            ],
            "id" => 1
        ];

        return $this->post($owsCall, false, false);
    }

    /**
     * Create candidate in OTYS database
     *
     * @return mixed
     *
     * @since 1.0.0
     */
    public function updateCandidate(string $candidateUID, array $candidateData)
    {
        $candidateData = $this->buildCandidateData($candidateData);

        $result = $this->post([
            "jsonrpc" => "2.0",
            "method" => "Otys.Services.CandidateService.updateEx",
            "params" => [
                $this->otysApi->session,
                $candidateUID,
                $candidateData
            ],
            "id" => 1
        ]);

        if ($result && ($return = $result)) {
            return $return;
        }

        return false;
    }

    /**
     * Get candidate based on email
     *
     * @param string $email
     * @return void
     * @since 1.0.0
     */
    public function getCandidate(string $email = null)
    {
        if ($email === null && is_email($email)) {
            return false;
        }

        $request = [
            "jsonrpc" => "2.0",
            "method" => "Otys.Services.CandidateService.getListEx",
            "params" => [
                $this->otysApi->session,
                [
                    'search' => [
                        'ACTONOMY' => [
                            'DATA' => [
                                'hasEmail' => [
                                    'options' => [
                                        'required' => true,
                                        'persistent' => true
                                    ],
                                    'value' => "1"
                                ],
                                'keywords' => [
                                    'options' => [
                                        'enableExpansion' => false,
                                        'required' => true,
                                        'searchList' => [
                                            'emailAddress'
                                        ],
                                        'searchMode' => 'one'
                                    ],
                                    "value" => $email
                                ]
                            ],
                            'OPTIONS' => [
                                'getTotalCount' => 1,
                                'limit' => 1,
                                'offset' => 0,
                                'sort' => [
                                    'SCORE' => 'DESC',
                                    'entryDateFull' => 'DESC'
                                ],

                            ],
                            "VERSION" => 2,
                            "SUB_VERSION" => 0
                        ]
                    ],
                    'limit' => 1,
                    'excludeLimitCheck' => 1,
                    'getTotalCount' => 1,
                    'what' => [
                        'uid' => 1
                    ]
                ]
            ],
            "id" => 1
        ];

        $candidateResult = $this->post($request);

        if (!empty($candidateResult) && !empty($candidate = $candidateResult['listOutput'])) {
            return $candidate[0];
        }

        return [];
    }

    /**
     * Check if a candidate exists based on email
     * @param string email adres of the candidate
     * @return boolean Returns true if email is found, false when email is not found
     *
     * @since 1.0.0
     */
    public function exists(string $email = null): bool
    {
        if ($email !== null && is_email($email)) {
            $candidate = $this->getCandidate($email);

            if (!empty($candidate)) {
                return true;
            }
        }

        return false;
    }

    /**
     * Build candidate data based on allowed OWS fields and returns candidateData
     * based on OWS names
     *
     * @return mixed
     * @since 1.0.0
     */
    public function buildCandidateData(array $candidateData): array
    {
        $candidate = [];
        $candidateObject = [];

        // Assign candidate data to candidate array and make changes to the data where needed
        foreach ($candidateData as $key => $value) {
            if ($key === 'Person.phoneMobile') {
                $candidate['phoneNumber'] = $value;
                continue;
            }

            $candidate[$key] = $value;
        }

        // After that we will make sure only allowed values are returned
        $candidate = array_filter($candidate, function ($key) {
            return in_array($key, $this->allowedUserInput);
        }, ARRAY_FILTER_USE_KEY);

        // Convert candidate data array to multidimensional array, since this is how OWS expect the data to be delivered
        foreach ($candidate as $owsName => $data) {
            $explode = explode('.', $owsName);
            $object = $this->flatArrayToMultidimensional($explode, $data);
            $candidateObject = array_merge_recursive($candidateObject, $object);
        }

        return $candidateObject;
    }

    /**
     * Create webuser
     *
     * @param string    $candidateUID Candidate UID
     * @param string    $email If email is not given a OWS request will be fired to
     *                  get the email adres from the candidate. If no email is found no webuser
     *                  will be created and the function will return empty.
     * @param string    Password needs to be string and more than 9 characters long otherwise a
     *                  auto password will be generated. Password will be included in the response
     * @return mixed    Returns the webuser id if succesfull or "already_exists" if
     *                  the webuser already exists.
     * @since 1.0.0
     */
    public function createWebuser(string $candidateUID, string $email = '', string $password ='')
    {
        // Sanatize email if email is given
        $email = $email !== '' ? sanitize_email($email) : $email;
        $password = $password !== '' && strlen($password) >= 9 ? $password : $this->createPassword();

        /**
         * If no email is given. Try to get the email adres from the candidate
         * using the candidate UID given. If this fails return empty and skip
         * creating a webuser
         */
        if ($email === '') {
            $candidateRequest = [
                "jsonrpc" => "2.0",
                "method" => "Otys.Services.CandidateService.getDetail",
                "params" => [
                    "{$this->otysApi->session}",
                    "{$candidateUID}",
                    [
                        "uid" => 1,
                        "Person" => [
                            "emailPrimary" => 1
                        ]
                    ],
                    null
                ],
                "id" => 1
            ];

            $candidateResponse = $this->post($candidateRequest, false, false);

            if (!empty($candidateResponse) && isset($candidateResponse['Person']) && isset($candidateResponse['Person']['emailPrimary'])) {
                $email = sanitize_email($candidateResponse['Person']['emailPrimary']);
            } else {
                return [];
            }
        }

        /**
         * Create webuser if an email is given
         */
        $createWebuserRequest = [
            "jsonrpc" => "2.0",
            "method" => "Otys.Services.OCDService.createWebuserAndBind",
            "params" => [
                "{$this->otysApi->session}",
                [
                    "role" => "candidate",
                    "uid" => "{$candidateUID}",
                    "username" => "{$email}",
                    "allowLogin" => true
                ]
            ],
            "id" => 1
        ];

        if (!empty($createWebuserResponse = $this->post($createWebuserRequest, false, false))) {
            $listWebusersRequest = [
                "jsonrpc" => "2.0",
                "method" => "Otys.Services.OCDService.getDetailByType",
                "params" => [
                    "{$this->otysApi->session}",
                    "candidate",
                    "uid" => "{$candidateUID}"
                ],
                "id" => 1
            ];

            $listWebusersResponse = $this->post($listWebusersRequest, false, false);

            if (!empty($listWebusersResponse) && is_array($listWebusersResponse)) {
                $webUserUid = array_search($email, $listWebusersResponse);

                if ($webUserUid) {
                    $setPasswordRequest = [
                        "jsonrpc" => "2.0",
                        "method" => "Otys.Services.WebusersService.setNewPassword",
                        "params" => [
                            "{$this->otysApi->session}",
                            "{$webUserUid}",
                            "{$password}"
                        ],
                        "id" => 1
                    ];

                    if (!empty($this->post($setPasswordRequest, false, false))) {
                        
                        return [
                            'webuser' => $createWebuserResponse,
                            'password' => $password
                        ];
                    }

                }
            }
        }

        return [];
    }

    /**
     * Generate random password
     *
     * @return void
     */
    public function createPassword()
    {
        $alphabet = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890';
        $pass = []; //remember to declare $pass as an array
        $alphaLength = strlen($alphabet) - 1; //put the length -1 in cache
        for ($i = 0; $i < 8; $i++) {
            $n = rand(0, $alphaLength);
            $pass[] = $alphabet[$n];
        }
        return implode($pass); //turn the array into a string
    }
}
