<?php

namespace Otys\OtysPlugin\Includes\OtysApi;

final class OtysApi extends OtysApiBase
{
    public $vacancies;
    public $matchCriteria;
    public $candidateQuestionSets;
    public $documents;
    public $candidates;
    public $settings;
    public $users;
    public $relations;
    public $procedures;
    public $session;
    public $mail;

    /**
     * apiInfo stores all base information about the current instance
     * of the API like which user is being used and for client the
     * current instance is being used
     *
     * @var array
     * @since 1.0.0
     */
    private $apiInfo = [];

    /**
     * Constructor
     *
     * @return void
     * @since 1.0.0
     */
    public function __construct($apiKey = '')
    {
        parent::__construct($this);

        $apiKey = $apiKey !== '' ? $apiKey : $this->getApiKey();

        $this->session = $this->createSession($apiKey);

        $this->initInfo();

        /**
         * Create instances of all functionalities being used
         */
        $this->users = new Users($this);
        $this->vacancies = new Vacancies($this);
        $this->matchCriteria = new MatchCriteria($this);
        $this->candidateQuestionSets = new CandidateQuestionsets($this);
        $this->documents = new Documents($this);
        $this->candidates = new Candidates($this);
        $this->settings = new Settings($this);
        $this->relations = new Relations($this);
        $this->procedures = new Procedures($this);
        $this->webhooks = new Webhooks($this);
        $this->mail = new Mail($this);
    }

    public function getSession(): string
    {
        return $this->session;
    }

    /**
     * setInfo sets the 
     *
     * @return void
     * @since 1.0.0
     */
    protected function initInfo()
    {
        $apiUser = $this->getApiUser();
        $websiteId = get_option('otys_option_website', 1);

        $this->apiInfo['websiteId'] = intval($websiteId);

        if ($apiUser) {
            $this->apiInfo['user'] = $apiUser;
            $this->apiInfo['clientCode'] = substr($apiUser['username'], strpos($apiUser['username'], '.') + 1);
            $this->apiInfo['clientId'] = $apiUser['clientId'];
        }
    }

    /**
     * getInfo returns information about the current instance of the
     * api, like which user is being used for communication and which
     * client the current instance of the api is being used for
     *
     * @param  string $key
     * @return mixed
     * @since 1.0.0
     */
    public function getInfo(string $key = NULL)
    {
        if ($key !== NULL && array_key_exists($key, $this->apiInfo)) {
            return $this->apiInfo[$key];
        } else if ($key === NULL) {
            return $this->apiInfo;
        }

        return false;
    }

    /**
     * Get the current logged in user. This returns the user which
     * is linked to the API Key. This user is used to do all the
     * OWS request.
     *
     * @return array
     * @since 1.0.0
     */
    public function getApiUser(): array
    {
        if (!$this->session) {
            return [];
        }

        $checkRequest = [
            "jsonrpc" => "2.0",
            "method" => "check",
            "params" => [
                $this->session
            ],
            "id" => 1
        ];

        if ($checkResponse = $this->post($checkRequest, true, false)) {
            $clientCodeRequest = [
                "jsonrpc" => "2.0",
                "method" => "Otys.Services.CsmService.getValue",
                "params" => [
                    $this->session,
                    [
                        "DB103"
                    ],
                    $checkResponse['clientId']
                ],
                "id" => 1
            ];
            
            if($clientCodeResponse = $this->post($clientCodeRequest, true, false)) {
                $checkResponse['clientCode'] = $clientCodeResponse['DB103']['value'];
            }

            return $checkResponse;
        }

        return [];
    }

    /**
     * Check if session is valid
     *
     * @return boolean
     */
    public function check(): bool
    {
        if (!$this->session) {
            return false;
        }

        $owsRequest = [
            "jsonrpc" => "2.0",
            "method" => "check",
            "params" => [
                $this->session
            ],
            "id" => 1
        ];

        $owsCall = $this->post($owsRequest, true, false);

        return (!empty($owsCall) ? true : false);
    }

    /**
     * Get the client code
     *
     * @return string
     * @since 1.0.0
     */
    public function getClientCode(): string
    {
        if (!empty($apiUser = $this->getApiUser()) && isset($apiUser['clientCode'])) {
            return $apiUser['clientCode'];
        }

        return '';
    }

    /**
     * Get the client code
     *
     * @return int
     * @since 1.0.0
     */
    public function getClientId(): int
    {
        $apiUser = $this->getApiUser();
        $clientId = $apiUser['clientId'];

        return $clientId;
    }
}
