<?php

namespace Otys\OtysPlugin\Includes\OtysApi;

use Otys\OtysPlugin\Includes\Core\Cache;
use Otys\OtysPlugin\Includes\Core\Logs;

abstract class OtysApiBase
{
    /**
     * The OTYS API Point
     *
     * @since 1.0.0
     */
    private $handler = "https://ows.otys.nl/jservice.php";
    // private $handler = "https://testing.otysapp.com/jservice.php";

    public $otysApi;

    public function __construct($otysApi)
    {
        $this->otysApi = $otysApi;
    }

    /**
     * getApiOptions
     * @return array
     * @since 1.0.0
     */
    public function getApiOptions()
    {
        return get_option('otys_settings');
    }

    /**
     * GET API key from options database
     *
     * @return string
     * @since 1.0.0
     */
    public function getApiKey()
    {
        return get_option('otys_option_api_key');
    }

    /**
     * Undocumented function
    *
    * @param mixed $request Array or JSON string request
    * @param mixed $useCache Use bool to use default values or specify a time in INT
    * @param boolean $useLanguageWrap Wheter to use the language wrap or not
    * @param array $args Arguments which get merged with OWS Call
    * @return mixed Returns either string or array depending on the request
    * @since 1.0.0
    */
    public function post($request, $cache = false, bool $useLanguageWrap = true, array $args = [])
    {
        // Init vars
        $requestMethod = $request['method'];
        $requestName = strtolower(str_replace('.', '_', str_replace('Otys.Services.', '', $requestMethod)));
        $originalRequest = $this->sanatizeRequest($request);
        $cacheTime = is_int($cache) && $cache > 1 ? $cache : 604800; // Default 7 days

        /**
         * Add language wrapper for Services requests. Only methods that contain
         * Services are supported by the language service therefore we will only
         * wrap those requests. Also prevent wrapping multiservice request since
         * that won't work.
         */
        if (
            ($useLanguageWrap === true || is_string($useLanguageWrap)) && 
            strpos($requestMethod, 'Otys.Services.') !== false && 
            $requestMethod !== 'Otys.Services.MultiService.execute'
        ) {
            $language = $this->getLanguage($useLanguageWrap);

            $request = [
                "jsonrpc" => "2.0",
                "method" => "Otys.Services.LanguageService.wrapRequest",
                "params" => [
                    $language,
                    $request
                ],
                "id" => 1
            ];
        }

        /**
         * Get cache for this request if useCache is true
         */
        if ($cache && $cachedData = Cache::get($requestName, $originalRequest)) {
            return $cachedData['value'];
        }

        // Create OWS request
        $owsRequest = wp_parse_args($args, [
            'method' => 'POST',
            'body' => (is_array($request) ? json_encode($request) : $request),
            'headers' => 'Content-Type: application/json\r\n' .
            'Accept: application/json\r\n',
            'timeout' => 5,
            'httpversion' => '1.0'
        ]);

        // Check there's cache for the current post
        $apiResult = wp_remote_post(
            $this->handler,
            $owsRequest
        );

        // Save log
        Logs::add($owsRequest, $apiResult);

        if (!empty($result = self::getResult($apiResult))) {

            /**
             * Save current result in cache if useCache is true
             */
            if ($cache) {
                Cache::add($requestName, $originalRequest, $result, $cacheTime);
            }

            return $result;
        }
        
        return [];
    }

    /**
     * Get the SESSION KEY that is used for all API Requests
     *
     * @param  string Optional key
     * @return mixed
     *
     * @since 1.0.0
     */
    public function createSession($key = false)
    {
        // If no key isset get a new key
        if ($key === false) {
            $key = $this->getApiKey();
        }

        if (!$key || $key === '' || strlen($key) < 8) {
            return false;
        }

        // We check for cache in the getSession so we can build in a language check in the current function
       
        // Get API Key using the clients API key
        $apiResult =  $this->post(
            [
                'jsonrpc' => '2.0',
                'method' => "loginByUid",
                'params' => [$key],
                'id' => 1
            ],
            28800 // 8 hours
        );

        if (!empty($apiResult)) {
            return $apiResult;
        }

        return false;
    }

    private function sanatizeRequest(array $request): string
    {
        $request = json_encode($request);

        $request = str_replace($this->otysApi->session, 'SESSION_ID_REPLACE', $request);
        
        return $request;
    }

    /**
     * Gets the body of a OTYS API Result
     *
     * @return mixed
     *
     * @since 1.0.0
     */
    public static function getBody($apiResult = null): array
    {
        if ($apiResult !== null && is_array($apiResult) && array_key_exists('body', $apiResult)) {
            $jsonBody = json_decode($apiResult["body"], true);

            if ($jsonBody === null) {
                return [];
            }
            return $jsonBody;
        }

        return [];
    }

    /**
     * Gets the result of the body of a OTYS API Result
     *
     * @return array
     * @since 1.0.0
     */
    public static function getResult($apiResult = null)
    {
        if (!empty($body = self::getBody($apiResult))) {
            if (array_key_exists('result', $body)) {
                return $body['result'];
            }
        }

        return [];
    }

    /**
     * Get OWS Fields translation for the VacancyService
     *
     * @param  array $fields expects OTYS database field names
     * @return array
     */
    public function getOwsNames(array $serviceNames, array $fields = []): array
    {
        $params = [];

        foreach ($serviceNames as $serviceName) {
            $params[] =  [
                "method" => "Otys.Services.{$serviceName}.getfieldTranslation",
                "args" => [
                    $this->otysApi->session,
                    "db",
                    $fields
                ]
            ];
        }

        $owsRequest = [
            "jsonrpc" => "2.0",
            "method" => "Otys.Services.MultiService.execute",
            "params" => [
                $params
            ],
            "id" => 1
        ];

        $owsCall = $this->post($owsRequest, true);

        return $owsCall;
    }

    /**
     * Get language based on current locale. Returns supported
     * language within OTYS
     *
     * @return string
     * @since 1.0.0
     */
    public function getLanguage($lang = ''): string
    {
        if ($lang === '' || !is_string($lang)) {
            $locale = explode('_', get_locale());
            $language = $locale[0];
        } else {
            $language = $lang;
        }

        $supportedLanguages = [
            'nl',
            'en',
            'de',
            'cs',
            'es',
            'fr'
        ];

        if (array_search($language, $supportedLanguages) !== false) {
            return $language;
        }

        return 'en';
    }

    /**
     * Create request from JSON with hidden SESSION ID
     *
     * @param string $request
     * @return array valid request with SESSION ID
     * @since 1.0.0
     */
    public function createRequest(string $request): array
    {
        $request = str_replace('SESSION_ID_REPLACE', $this->otysApi->session, $request);

        $request = json_decode($request, true);

        return $request;
    }

     /**
     * flatArrayToMultidimensional
     *
     * @param  mixed $array
     * @param  mixed $lastKeyValue
     * @return array
     * @since 1.0.0
     */
    public function flatArrayToMultidimensional(array $array, $lastKeyValue = []): array {
        $lastKeyValue = json_encode($lastKeyValue);   
        
        /**
         * We will be using JSON to build the new array since we can easily build
         * a string based object. Later on we will use this JSON to convert it
         * to an array and return a error
         */
        $output = '{';
        
        // Loop through the array
        foreach ($array as $arrayKey => $value) {
            /**
             * Check if the current key is the last key since when the current
             * key is the last key we want to assign the user given value to
             * the last property
             */
            if (array_key_last($array) === $arrayKey) {
                // Assign the user given value to the last property
                $output .= '"' . $value . '" : ' . $lastKeyValue;
            } else {
                // Create a new layer in the JSON
                $output .= '"' . $value . '" : {';
            }
        }
        
        /**
         * Since we didn't close the curly brackets yet we will need to close
         * every layer we created. So we will again loop through the array and
         * add a closing curly bracket for every layer.
         */
        foreach ($array as $arrayKey => $value) {
            $output .= '}';
        }
        
        // Conver the json to an array and return it
        return json_decode($output, true);
    }

    /**
     * Get current multisite website ID
     *
     * @return integer Returns 0 when no there's no multisite environment.
     * This is because if there are no multiple websites, you should not use
     * the site id in calls.
     * @since 1.1.2
     */
    public function getCurrentMultisiteID(): int
    {
        $websiteOption = intval(get_option('otys_option_website', 0));
        $websites = $this->getwebsites();

        // If there's only one website return 0 because there is no multisite
        if (count($websites) <= 1 && $websiteOption !== 0) {
            // If current website option is not 0 and there are no multisites we update the option aswell
            if ($websiteOption >= 1) {
                update_option('otys_option_website', 0);
            }

            return 0;
        }
        
        return $websiteOption;
    }

    /**
     * Get list of websites
     *
     * @return array
     * @since 1.0.0
     */
    public function getWebsites(): array
    {
        $owsRequest = [
            "jsonrpc" => "2.0",
            "method" => "Otys.Services.VacancyService.getOptionLists",
            "params" => [
                $this->otysApi->session,
                [
                    "publishedWebsites"
                ]
            ],
            "id" => 1
        ];

        $owsCall = $this->post($owsRequest, true, false);

        $owsResult = $owsCall;

        if (!empty($owsResult)) {
            return $owsResult['publishedWebsites'];
        }

        return [];
    }
}
