<?php

namespace Otys\OtysPlugin\Models;

use Otys\OtysPlugin\Includes\OtysApi\OtysApi;

abstract class BaseModel extends \Otys\OtysPlugin\Includes\Core\Base
{
    public $modulename = 'unset';

    /**
     * Holds the args that are parsed to the template using the Wordpress load_template function
     *
     * @var array
     * @since 1.0.0
     */
    private $args = array();

    /**
     * getArgs
     *
     * @since 1.0.0
     * @return void
     */
    public function getArgs()
    {
        return $this->args;
    }

    /**
     * Create new instance of the API
     *
     * @return object
     * @since 1.0.0
     */
    public function otysApi(): object
    {
        return new OtysApi();
    }

    /**
     * Parse args to template using the Wordpress load_template function
     *
     * The loadTemplate function uses the wordpress load_template function which allows to to parse
     * args to the template to load. Using parseArgs() we are saving args to the $this->args array
     * which we can later on parse to the template.
     *
     * @param string $name Name of argument this is later on used as $args[$name] in the template
     * @param mixed $data The data you want to parse to the template
     * @return void
     * @since 1.0.0
     */
    public function parseArgs(string $name, $data): void
    {
        $this->args[$name] = $data;
    }

    /**
     * Load template using the Wordpress load_template function
     *
     * @return bool
     * @since 1.0.0
     */
    public function loadTemplate(): bool
    {
        if ($template = $this->getTemplate()) {
            load_template($template, true, $this->args);

            return true;
        }

        return false;
    }

    /**
     * Generate pagination array
     *
     * @param  int $maxItems
     * @param  int $totalItems
     * @return array
     * @since 1.0.0
     */
    public function getPagination(int $maxItems, int $totalItems, int $currentPage, array $allowedParams = []): array
    {
        // Set default allowed params
        $allowedParams = wp_parse_args($allowedParams, [
            'page-number',
            'search'
        ]);

        // Sanatize $_GET parameters
        $getParams = array_map(function($value) {
            return htmlspecialchars($value, ENT_QUOTES);
        }, $_GET);

        // Filter allowed parameters
        $params = array_filter($getParams, function($key) use ($allowedParams) {
            return (array_search($key, $allowedParams) !== false);
        }, ARRAY_FILTER_USE_KEY);

        // Set total page items
        $totalPages = intval((ceil($totalItems / $maxItems)));

        // Initialiase pagination array
        $pagination = [
            'pages' => [],
            'total_pages' => (int) $totalPages,
            'current_page_number' => (int) $currentPage
        ];

        // Get current url
        global $wp;
        $currentUrl = home_url(add_query_arg(array(), $wp->request));

        // Create pages array
        for ($i = 1; $i <= $totalPages; $i++) {
            $pageQuery = $params;

            // If the page is 1 remove the page-number parameter from the url
            if ($i !== 1) {
                $pageQuery['page-number'] = $i;
            } else {
                unset($pageQuery["page-number"]);
            }

            $httpPageQuery = http_build_query($pageQuery);

            $pagination['pages'][] = [
                'active' => ($currentPage === $i) ? true : false,
                'url' => ($httpPageQuery !== "") ? $currentUrl . '?' . http_build_query($pageQuery) : $currentUrl,
                'page' => $i
            ];
        }

        // Get current get parameters which we will edit parameters after that build
        $prevPageQuery = $params;

        // If the page is 1 remove the page-number parameter from the url
        if (($prevPageNumber = ($currentPage - 1)) !== 1) {
            $prevPageQuery['page-number'] = (int) $prevPageNumber;
        } else {
            unset($prevPageQuery["page-number"]);
        }

        // Build the query parameters for the prev link
        $httpPrevQuery = http_build_query($prevPageQuery);
        $prevUrl = ($httpPrevQuery !== "") ? $currentUrl . '?' . $httpPrevQuery : $currentUrl;
        $pagination["prev"] = [
            "url" => sanitize_url($prevUrl),
            "show" => ($currentPage !== 1 && $totalPages !== 1) ? true : false
        ];

        // Create next link
        $nextPageQuery = $params;
        $nextPageQuery['page-number'] = intval($currentPage) + 1;
        $pagination["next"] = [
            "url" => sanitize_url($currentUrl . '?' . http_build_query($nextPageQuery)),
            "show" => ($currentPage < $totalPages && intval($currentPage + 1) <= $totalPages) ? true : false
        ];

        return (array) $pagination;
    }
}
