<?php

namespace Otys\OtysPlugin\Models;

use Otys\OtysPlugin\Includes\Core\Cache;
use Otys\OtysPlugin\Includes\OtysApi\OtysApi;

class VacanciesModel extends BaseModel
{

    /**
     * Stores vacancies result from cache or ows
     *
     * @var mixed
     *
     * @since 1.0.0
     */
    protected $vacancies;

    /**
     * Stores list of selected match criteria
     *
     * @var mixed
     *
     * @since 1.0.0
     */
    protected $selectedMatchCriteria;

    /**
     * Get Matchcriteria list slug based
     *
     * @return array
     * @param array $criteria List of criteria to display
     *
     * @since 1.0.0
     */
    public function getMatchCriteriaSlugs($matchCriteriaToShow = null)
    {        
        /**
         * When we have no cache
         */
        $otysApi = new OtysApi();
        
        $matchCriteriaList = $otysApi->matchCriteria->getListSlugs($matchCriteriaToShow);

        return $matchCriteriaList;
    }

    /**
     * Get selected match criteria array to be used in API Call
     *
     * @return array
     *
     * @since 1.0.0
     */
    public function getSelectedMatchCriteria($args = []): array
    {
        $matchCriteriaSlugs = $this->getMatchCriteriaSlugs();
        $getParameters = $this->getQueryParameters($args);

        // Generate a selected match criteria list
        $selectedMatchCriteria = [];

        // Loop through match criteria slugged and save the ones that are set as $_GET parameter
        foreach ($matchCriteriaSlugs as $criteriaSlug => $criteria) {
            if (isset($getParameters[$criteriaSlug])) {
                $selectedMatchCriteria[$criteria['id']]['options']['required'] = true;
                $selectedMatchCriteria[$criteria['id']]['value']['required'] = [];

                // Get options seperate options with plus sign
                $matchCriteriaUrlOptions = explode("+", urlencode($getParameters[$criteriaSlug]));

                // Loop through all options and check if the option is within the url
                foreach ($matchCriteriaUrlOptions as $selectedOption) {
                    if (isset($criteria['options'][$selectedOption]) && array_key_exists('id', $criteria['options'][$selectedOption])) {
                        $selectedMatchCriteria[$criteria['id']]['value']['required'][] = $criteria['options'][$selectedOption]['id'];
                    }
                }
            }
        }

        return $selectedMatchCriteria;
    }

    /**
     * Get matchcriteria list which will be used in the front end
     * The matchcriteria list will contain which options / criteria are active & frequency numbers
     *
     * @return array
     *
     * @since 1.0.0
     */
    public function getMatchCriteriaList($selectedCriteria = []): array
    {
        // Get GET paramaters
        $getParameters = $this->getQueryParameters($selectedCriteria);

        // Get the list of criteria we want to show
        $matchCriteriaToShow = get_option('otys_option_vacancies_filters_match_criteria');

        $matchCriteriaSlugs = $this->getMatchCriteriaSlugs($matchCriteriaToShow);

        // Generate new Matchcriteria list so we can edit the list
        $matchCriteriaResult = $matchCriteriaSlugs;

        // Validate query parameters and perform action accodingly
        foreach ($getParameters as $getKey => $get_value) {
            if (empty($get_value)) {
                // If parameter if empty remove the parameter
                unset($getParameters[$getKey]);
            }
        }

        // Loop through all matchcriteria
        foreach ($matchCriteriaSlugs as $criteriaSlug => $criteria) {
            // Generate new GET URL

            if (isset($getParameters[$criteriaSlug])) {
                // Get options seperate options with plus sign
                $getOptions = explode("+", urlencode($getParameters[$criteriaSlug]));
                $getOptions_count = count($getOptions);
            } else {
                $getOptions = false;
            }

            // Set active status current criteria
            if (array_key_exists($criteriaSlug, $getParameters)) {
               $matchCriteriaResult[$criteriaSlug]['active'] = true;
            }

            // SET URL By Looping through all options of the current matchcriteria and deciding what the url must be
            foreach ($criteria['options'] as $criteriaOptionSlug => $criteria_option) {
                // Set new_get variable to generate url for this option
                $newUrl = $getParameters;

                // Set initial value for url, this should never result in nothing, but just to make sure to return something
                $url = '';

                if (!isset($getParameters)) {
                    // When there is no get set we can expect the url to be as simple as ?criterianame=option
                    $url = '?' . $criteriaSlug . '=' . $criteriaOptionSlug;
                } elseif ($getOptions !== false && array_key_exists($criteriaSlug, $getParameters)) {
                    // There are options set for the current matchcriteria

                    // Set a new options variable which we will use to generate a new options string
                    $newoptions = $getOptions;

                    // Check if the current option is within the GET
                    if (($option_key = array_search($criteriaOptionSlug, $newoptions)) !== false) {
                        // Since the option is currently selected we want to deselect the option and therefore we remove the option from the optionlist
                        unset($newoptions[$option_key]);

                        // Since the current option is selected we also want to add the active status to the current option in the match criteria list
                        $matchCriteriaResult[$criteriaSlug]['options'][$criteriaOptionSlug]['active'] = true;
                    } else {
                        // Since the curent option is not currently selected we want to add the option to the option list
                        $newoptions[] = $criteriaOptionSlug;
                    }

                    // Decide wheter to keep the current matchcriteria in the new url
                    if (empty($newoptions)) {
                        // If the new option list will be empty remove the entire matchcriteria
                        unset($newUrl[$criteriaSlug]);
                    } else {
                        // If there are still options create the new options list
                        $newUrl[$criteriaSlug] = implode(" ", $newoptions);
                    }
                } else {
                    // The current matchcriteria is not yet in the url
                    $newUrl[$criteriaSlug] = $criteriaOptionSlug;
                }

                // Place search parameter at the end of the url
                if (!empty($newUrl['search'])) {
                    $searchValue = $newUrl['search'];
                    unset($newUrl['search']);
                    $newUrl['search'] = $searchValue;
                }

                // Build new url get parameters
                $url = http_build_query($newUrl);

                // Assign generated url to the current option
                $matchCriteriaResult[$criteriaSlug]['options'][$criteriaOptionSlug]['url'] = $url;
            }
        }

        return $matchCriteriaResult;
    }

    /**
     * Get a list of allowed query parameters
     *
     * @return array
     *
     * @since 1.0.0
     */
    public function getQueryParameters($selectedCriteria = []): array
    {
        // Skip get parameters and return empty array
        if ((!isset($_GET) || empty($_GET)) && empty($selectedCriteria)) {
            return [];
        }

        $matchCriteriaSlugged = $this->getMatchCriteriaSlugs();
        $return = [];
        
        /**
         * Match criteria in url
         */
        foreach ($matchCriteriaSlugged as $matchCriteriaSlug => $matchCriteria) {
            // Check if current match criteria is in selected criteria
            if (
                array_key_exists($matchCriteria['id'], $selectedCriteria) ||
                array_key_exists($matchCriteriaSlug, $selectedCriteria)
            ) {
                $selectedCriteriaFiltered = [];

                foreach ($matchCriteria['options'] as $optionSlug => $option) {
                    if (
                        (isset($selectedCriteria[$matchCriteria['id']]) && 
                        in_array($option['id'], $selectedCriteria[$matchCriteria['id']]))
                        ||
                        (isset($selectedCriteria[$matchCriteriaSlug]) &&
                        in_array($optionSlug, $selectedCriteria[$matchCriteriaSlug]))
                    ) {
                        $selectedCriteriaFiltered[] = $optionSlug;
                    }
                }

                $return[$matchCriteriaSlug] = implode(" ", $selectedCriteriaFiltered);

                continue;
            }

            if (isset($_GET[$matchCriteriaSlug])) {
                $getParameterOptions = explode("+", urlencode(htmlspecialchars($_GET[$matchCriteriaSlug], ENT_QUOTES)));

                // Check if the options in the url exist as criteria, if not delete them
                foreach ($getParameterOptions as $parameterOptionKey => $parameterOption) {
                    if (!array_key_exists($parameterOption, $matchCriteriaSlugged[$matchCriteriaSlug]['options'])) {
                        unset($getParameterOptions[$parameterOptionKey]);
                    }
                }

                $return[$matchCriteriaSlug] = implode(" ", $getParameterOptions);
            }
        }

        /**
         * Search in url
         */
        if (isset($_GET['search']) && !empty($_GET['search']) && is_string($_GET['search'])) {
            $return['search'] = trim(preg_replace('/[^a-zA-Z0-9 .,&\*\']/', '', $_GET['search']));
        }
        

        return $return;
    }

    /**
     * Get selected criteria list with links
     *
     * @return void
     *
     * @since 1.0.0
     */
    public function getSelectedCriteria($selectedCriteria = []): array
    {
        if ($this->selectedMatchCriteria !== null) {
            return $this->selectedMatchCriteria;
        }

        $queryParameters = $this->getQueryParameters($selectedCriteria);
        $matchCriteriaSlugs = $this->getMatchCriteriaSlugs();

        // List of selected parameters that we will build and later return
        $selectedParameters = [];

        // remove page from query parameters since we don't want to stay on the same page when removing a parameter
        unset($queryParameters['page-number']);

        // Loop through all query parameters and build the selected criteria list
        foreach ($queryParameters as $parameterName => $parameterValue) {
            // Check if parameter name and option exist
            $isCriteria = array_key_exists($parameterName, $matchCriteriaSlugs) && array_key_exists($parameterValue, $matchCriteriaSlugs[$parameterName]);

            // If parameter is not a matchcriteria or search keyword skip the current parameter
            if (!$isCriteria && $parameterName !== 's') {
                continue;
            }

            $newQueryParameters = $queryParameters;

            // Remove the current parameter from the url to generate the new url without the parameter
            unset($newQueryParameters[$parameterName]);

            // Set new url
            $selectedParameters[$parameterName]['url'] = http_build_query($newQueryParameters);

            if ($isCriteria) {
                // If is criteria return create names belonging to the slug
                $slugValues = explode(" ", $parameterValue);
                $nameValues = [];

                foreach ($slugValues as $option_slug) {
                    $nameValues[] = $matchCriteriaSlugs[$parameterName]['options'][$option_slug]['name'];
                }

                if (!empty($nameValues)) {
                    $selectedParameters[$parameterName]['name'] = $nameValues;
                    $selectedParameters[$parameterName]['type'] = 'match_criteria';
                }
            } elseif ($parameterName === 's') {
                // If is search parameter return search value
                $selectedParameters[$parameterName]['name'][] = $parameterValue;
                $selectedParameters[$parameterName]['type'] = 'search';
            }
        }

        $this->selectedMatchCriteria = $selectedParameters;

        return $selectedParameters;
    }
}